#!/bin/bash

#
# Copyright (c) 2019-2021, NVIDIA CORPORATION.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of NVIDIA CORPORATION nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

if [ ! -e /etc/nv/nvfirstboot ]; then
	exit 0
fi

function wait_debconf_resource() {
	# Wait for the process to finish which has aquired this lock
	while fuser "/var/cache/debconf/config.dat" > "/dev/null" 2>&1; do sleep 1; done;
	while fuser "/var/cache/debconf/templates.dat" > "/dev/null" 2>&1; do sleep 1; done;
}

update-alternatives \
	--install /etc/ld.so.conf.d/aarch64-linux-gnu_EGL.conf \
	aarch64-linux-gnu_egl_conf \
	/usr/lib/aarch64-linux-gnu/tegra-egl/ld.so.conf 1000
update-alternatives \
	--install /etc/ld.so.conf.d/aarch64-linux-gnu_GL.conf \
	aarch64-linux-gnu_gl_conf \
	/usr/lib/aarch64-linux-gnu/tegra/ld.so.conf 1000

if [ ! -e "/usr/bin/python" ] && [ ! -e "/etc/alternatives/python" ]; then
	if [ -e "/usr/bin/python2" ]; then
		update-alternatives --install /usr/bin/python python /usr/bin/python2 1000
	elif [ -e "/usr/bin/python3" ]; then
		update-alternatives --install /usr/bin/python python /usr/bin/python3 1000
	fi
fi

CHIP="$(cat /proc/device-tree/compatible)"
if [[ ${CHIP} =~ "tegra186" ]] || [[ ${CHIP} =~ "tegra210" ]] || [[ ${CHIP} =~ "tegra194" ]]; then
	ARCH=$(/usr/bin/dpkg --print-architecture)
	if [ "${ARCH}" = "arm64" ]; then
		LIB_DIR="/usr/lib/aarch64-linux-gnu"
	fi
	if [ -e "/dev/tegra_dc_ctrl" ] && [ -e "${LIB_DIR}/tegra/libdrm.so.2" ]; then
		sudo bash -c "ln -sf tegra/libdrm.so.2 ${LIB_DIR}/libdrm_nvdc.so"
	fi
fi

ldconfig

# Reconfigure blueman if it's present in rootfs
if [ -d "/usr/lib/blueman" ]; then
	wait_debconf_resource
	dpkg-reconfigure blueman
fi

# Read total memory size in megabyte
if TOTAL_MEM=$(free --mega | awk '/^Mem:/{print $2}'); then
	TOTAL_MEM=$(echo "scale=1;${TOTAL_MEM}/1000" | bc)
	TOTAL_MEM=$(echo "${TOTAL_MEM}" | awk '{print int($1+0.5)}')

	# If RAM size is less than 4 GB, set default display manager as LightDM
	if [ -e "/lib/systemd/system/lightdm.service" ] &&
		[ "${TOTAL_MEM}" -lt 4 ]; then
		DEFAULT_DM=$(cat "/etc/X11/default-display-manager")
		if [ "${DEFAULT_DM}" != "/usr/sbin/lightdm" ]; then
			echo "/usr/sbin/lightdm" > "/etc/X11/default-display-manager"
			wait_debconf_resource
			DEBIAN_FRONTEND=noninteractive DEBCONF_NONINTERACTIVE_SEEN=true dpkg-reconfigure lightdm
			echo set shared/default-x-display-manager lightdm | debconf-communicate
		fi
	fi
else
	echo "ERROR: Cannot get total memory size."
fi

# Allow anybody to run X
if [ -f "/etc/X11/Xwrapper.config" ]; then
	sed -i 's/allowed_users.*/allowed_users=anybody/' "/etc/X11/Xwrapper.config"
	echo "needs_root_rights=yes" >> "/etc/X11/Xwrapper.config"
fi

groupadd -rf debug # Add debug group - http://nvbugs/2823941

# WAR (to be fixed in http://nvbugs/200640832): udev and this script will have synchronization
# problem in the first run, manually chown until the bug is fixed.

if [ -e /dev/nvhost-ctxsw-gpu ];then
    chown root:debug /dev/nvhost-ctxsw-gpu
fi
if [ -e /dev/nvhost-dbg-gpu ];then
    chown root:debug /dev/nvhost-dbg-gpu
fi
if [ -e /dev/nvhost-prof-ctx-gpu ];then
    chown root:debug /dev/nvhost-prof-ctx-gpu
fi
if [ -e /dev/nvhost-prof-dev-gpu ];then
    chown root:debug /dev/nvhost-prof-dev-gpu
fi
if [ -e /dev/nvhost-prof-gpu ];then
    chown root:debug /dev/nvhost-prof-gpu
fi
if [ -e /dev/nvhost-sched-gpu ];then
    chown root:root /dev/nvhost-sched-gpu
fi
if [ -e /dev/nvgpu/igpu0/ctxsw ];then
    chown root:debug /dev/nvgpu/igpu0/ctxsw
fi
if [ -e /dev/nvgpu/igpu0/dbg ];then
    chown root:debug /dev/nvgpu/igpu0/dbg
fi
if [ -e /dev/nvgpu/igpu0/prof-ctx ];then
    chown root:debug /dev/nvgpu/igpu0/prof-ctx
fi
if [ -e /dev/nvgpu/igpu0/prof-dev ];then
    chown root:debug /dev/nvgpu/igpu0/prof-dev
fi
if [ -e /dev/nvgpu/igpu0/prof ];then
    chown root:debug /dev/nvgpu/igpu0/prof
fi
if [ -e /dev/nvgpu/igpu0/sched ];then
    chown root:root /dev/nvgpu/igpu0/sched
fi
